/*
 *		Copyright IBM Corporation 1989
 *
 *                      All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of IBM not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 *
 * University of Illinois at Urbana-Champaign
 * Department of Computer Science
 * 1304 W. Springfield Ave.
 * Urbana, IL	61801
 *
 * (C) Copyright 1987, 1988 by The University of Illinois Board of Trustees.
 * All rights reserved.
 *
 * Tool: X 11 Graphical Kernel System
 * Author: Gregory Scott Rogers
 * Author: Sung Hsien Ching Kelvin
 * Author: Yu Pan
 * 
 * This file is the header-file for the XGKS implementation.  It includes most
 * necessary X header-files and most XGKS header-files -- it does not include
 * some "specialty" header-files or those specific to the Fortran 
 * implementation).  All XGKS library implementation-files should include this
 * header-file.
 *
 * This header-file should be included before any "specialty" X header-files
 * that depend upon <X11/Xlib.h>.
 *
 * $Id: gks_implem.h,v 2.5.4.1 1993/03/10 19:44:21 steve Exp $
 * $__Header$
 */

#ifndef XGKS_GKS_IMPLEM_H
#define XGKS_GKS_IMPLEM_H

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include "xgks.h"
#include "gks_defines.h"
#include "primitive.h"
#include "input.h"
#include "wdt.h"
#include "wslist.h"
#include "gkslist.h"
#include "gks_errors.h"
#include "event.h"
#include "metafile.h"

/* We now enable -Werror=implicit-function-declaration on Debian
 * so we need to include these prototypes.
 * Not necessarily the exported API
 */

Gint gopenws (Gint wsid,const Gchar *connection,const Gchar *ws_type);
Gint gclosews (Gint wsid);
Gint gclearws (Gint wsid, Gclrflag control_flag);
Gint gopengks (Gfile *err_file, Glong memory);
Gint gclosegks (void);
Gint gsetasf (Gasfs *asf);
Gint gsetcolorrep (Gint wsid, Gint idx, Gcobundl *rep);
Gint ginqcolorrep (Gint wsid, Gint idx, Gqtype type, Gcobundl *rep);
Gint ginqcolorindices (Gint wsid, Gintlist *indices);
Gint ginqcolorfacil (Gchar *ws_type, Gcofac *fac);
Gint ginqpredcolorrep (Gchar *ws_type, Gint index, Gcobundl *rep);
Gint gactivatews (Gint wsid);
Gint gdeactivatews (Gint wsid);
Gint gsetwindow (Gint trans, Glimit *window);
Gint gsetviewport (Gint trans, Glimit *viewport);
Gint gsetviewportinputpri (Gint trans, Gint ref_trans, Gvpri priority);
Gint gselntran (Gint trans);
Gint gsetclip (Gclip ind);
Gint gsetwswindow (Gint wsid, Glimit *window);
Gint gsetwsviewport (Gint wsid, Glimit *viewport);
Gint gevaltran (Gpoint *ppoint, Gpoint *pshift, Gfloat angle, Gpoint *pscale, Gcsw coord, Gfloat result[2][3]);
Gint gaccumtran (Gfloat segtran[2][3], Gpoint *ppoint, Gpoint *pshift, Gfloat angle, Gpoint *pscale, Gcsw coord, Gfloat result[2][3]);
Gint gemergencyclosegks (void);
Gint gerrorhand (Gint errnum, Gerrmap funcname, Gfile *perrfile);
Gint gerrorlog (Gint errnum, Gerrmap funcname, Gfile *perrfile);
Gint gpolyline (Gint num_pts, Gpoint *pts);
Gint gsetlineind (Gint idx);
Gint gsetlinetype (Gint type);
Gint gsetlinewidth (Gfloat width);
Gint gsetlinecolorind (Gint idx);
Gint gsetlinerep (Gint wsid, Gint idx, Glnbundl *rep);
Gint gcreateseg (Gint name);
Gint gcloseseg (void);
Gint grenameseg (Gint old, Gint new);
Gint gdelseg (Gint name);
Gint gdelsegws (Gint wsid, Gint name);
Gint gsetsegattr (Gint name, Gsegattr *segattr);
Gint gsetpickid (Gint pick_id);
Gint gassocsegws (Gint wsid, Gint seg_id);
Gint gcopysegws (Gint wsid, Gint seg_id);
Gint ginsertseg (Gint seg_id, Gfloat segtran[2][3]);
Gint gredrawsegws (Gint wsid);
Gint ginqmodsegattr (Gchar *ws_type, Gmodseg *dyn);
Gint ginqnameopenseg (Gint *seg);
Gint ginqsegattr (Gsegattr *segattr);
Gint ginqassocws (Gint seg, Gintlist *asswk);
Gint ginqsegnames (Gintlist *segs);
Gint ginqsegnamesws (Gint wsid, Gintlist *segs);
Gint ginqnumsegpri (Gchar *ws_type, Gint *numpri);
Gint ginqcurpickid (Gint *pickid);
Gint gpolymarker (Gint num_pts, Gpoint *pts);
Gint gsetmarkersize (Gfloat size);
Gint gsetmarkertype (Gint type);
Gint gsetmarkercolorind (Gint color);
Gint gsetmarkerind (Gint idx);
Gint gsetmarkerrep (Gint wsid, Gint idx, Gmkbundl *rep);
Gint gfillarea (Gint num_pts, Gpoint *pts);
Gint gsetfillcolorind (Gint color);
Gint gsetfillind (Gint idx);
Gint gsetfillintstyle (Gflinter style);
Gint gsetfillrep (Gint wsid, Gint idx, Gflbundl *rep);
Gint gsetfillstyleind (Gint idx);
Gint gsetpatrep (Gint wsid, Gint idx, Gptbundl *rep);
Gint gcellarray (Grect *rect, Gipoint *dim, Gint row, Gint *color);
Gint ginitloc (Gint wsid, Gint dev, Gloc *init, Gint pet, Glimit *area, Glocrec *record);
Gint gsetlocmode (Gint wsid, Gint dev, Gimode mode, Gesw echo);
Gint greqloc (Gint wsid, Gint dev, Gqloc *response);
Gint gsampleloc (Gint wsid, Gint dev, Gloc *response);
Gint ginqlocst (Gint wsid, Gint dev, Gqtype type, Glocst *state);
Gint ginqdefloc (Gchar *type, Gint dev, Gdefloc *data);
Gint gawaitevent (Gfloat timeout, Gevent *event);
Gint gflushevents (Gint wsid, Giclass class, Gint dev);
Gint ginqinputoverflow (Gqueue *overflow);
Gint ginqmoreevents (Gsimultev *events);
Gint ggetloc (Gloc *response);
Gint ggetstroke (Gstroke *response);
Gint ggetchoice (Gchoice *response);
Gint ggetpick (Gpick *response);
Gint ggetval (Gfloat *response);
Gint ggetstring (Gchar *response);
Gint gupdatews (Gint wsid, Gregen regenflag);
Gint gsetcharexpan (Gfloat expansion);
Gint gsetcharheight (Gfloat height);
Gint gsetcharspace (Gfloat spacing);
Gint gsetcharup (Gpoint *up_vector);
Gint gsettextalign (Gtxalign *txalign);
Gint gsettextcolorind (Gint color);
Gint gsettextfontprec (Gtxfp *txfp);
Gint gsettextind (Gint idx);
Gint gsettextpath (Gtxpath path);
Gint gsettextrep (Gint wsid, Gint idx, Gtxbundl *bundle);
Gint gtext (Gpoint *at, Gchar *string);
Gint ginqtextextent (Gint wsid, Gpoint position, Gchar *string, Gextent *extent);
Gint gsetdeferst (Gint wsid, Gdefmode deferral_mode, Girgmode regen_mode);
Gint ginitchoice (Gint wsid, Gint dev, Gchoice *init, Gint pet, Glimit *area, Gchoicerec *record);
Gint gsetchoicemode (Gint wsid, Gint dev, Gimode mode, Gesw echo);
Gint greqchoice (Gint wsid, Gint dev, Gchoice *response);
Gint gsamplechoice (Gint wsid, Gint dev, Gchoice *response);
Gint ginqchoicest (Gint wsid, Gint dev, Gchoicest *state);
Gint ginqdefchoice (Gchar *type, Gint dev, Gdefchoice *data);
Gint ginitpick (Gint wsid, Gint dev, Gpick *init, Gint pet, Glimit *area, Gpickrec *record);
Gint gsetpickmode (Gint wsid, Gint dev, Gimode mode, Gesw echo);
Gint greqpick (Gint wsid, Gint dev, Gpick *response);
Gint gsamplepick (Gint wsid, Gint dev, Gpick *response);
Gint ginqpickst (Gint wsid, Gint dev, Gqtype type, Gpickst *state);
Gint ginqdefpick (Gchar *type, Gint dev, Gdefpick *data);
Gint ginitval (Gint wsid, Gint dev, Gfloat init, Gint pet, Glimit *area, Gvalrec *record);
Gint gsetvalmode (Gint wsid, Gint dev, Gimode mode, Gesw echo);
Gint greqval (Gint wsid, Gint dev, Gqval *response);
Gint gsampleval (Gint wsid, Gint dev, Gfloat *response);
Gint ginqvalst (Gint wsid, Gint dev, Gvalst *state);
Gint ginqdefval (Gchar *type, Gint dev, Gdefval *data);
Gint ginitstring (Gint wsid, Gint dev, Gchar *init, Gint pet, Glimit *area, Gstringrec *record);
Gint gsetstringmode (Gint wsid, Gint dev, Gimode mode, Gesw echo);
Gint greqstring (Gint wsid, Gint dev, Gqstring *response);
Gint gsamplestring (Gint wsid, Gint dev, Gchar *response);
Gint ginqstringst (Gint wsid, Gint dev, Gstringst *state);
Gint ginqdefstring (Gchar *type, Gint dev, Gdefstring *data);
Gint ginitstroke (Gint wsid, Gint dev, Gstroke *init, Gint pet, Glimit *area, Gstrokerec *record);
Gint gsetstrokemode (Gint wsid, Gint dev, Gimode mode, Gesw echo);
Gint greqstroke (Gint wsid, Gint dev, Gqstroke *response);
Gint gsamplestroke (Gint wsid, Gint dev, Gstroke *response);
Gint ginqstrokest (Gint wsid, Gint dev, Gqtype type, Gstrokest *state);
Gint ginqdefstroke (Gchar *type, Gint dev, Gdefstroke *data);
Gint ginqlinefacil (Gchar *ws_type, Glnfac *fac);
Gint ginqpredlinerep (Gchar *ws_type, Gint idx, Glnbundl *rep);
Gint ginqlineindices (Gint wsid, Gintlist *idxlist);
Gint ginqlinerep (Gint wsid, Gint idx, Gqtype type, Glnbundl *rep);
Gint ginqmarkerfacil (Gchar *ws_type, Gmkfac *fac);
Gint ginqpredmarkerrep (Gchar *ws_type, Gint idx, Gmkbundl *rep);
Gint ginqmarkerindices (Gint wsid, Gintlist *idxlist);
Gint ginqmarkerrep (Gint wsid, Gint idx, Gqtype type, Gmkbundl *rep);
Gint ginqfillfacil (Gchar *ws_type, Gflfac *fac);
Gint ginqpredfillrep (Gchar *ws_type, Gint idx, Gflbundl *rep);
Gint ginqfillindices (Gint wsid, Gintlist *idxlist);
Gint ginqfillrep (Gint wsid, Gint idx, Gqtype type, Gflbundl *rep);
Gint ginqpatfacil (Gchar *ws_type, Gint *fac);
Gint ginqpredpatrep (Gchar *ws_type, Gint idx, Gptbundl *rep);
Gint ginqpatindices (Gint wsid, Gintlist *idxlist);
Gint ginqpatrep (Gint wsid, Gint idx, Gqtype type, Gptbundl *rep);
Gint gmessage (Gint wsid, Gchar *string);
Gint gescinqxattr (Gint wsid, Display **dpy, Window *win, GC *gc);
void gescsetbackingstore (Gint wsid, Gint i);
void gescsetprogname (char *name);
Gint gescsetcolormask (Gint wsid, unsigned long mask);
Gint gescsetdcsize (Gint wsid, Gpoint size);
Gint gescstoreprimi (Gint wsid, Gstore store);
Gint gescredrawnotify (Gint wsid, Gint (*funcp)(Gint wsid, Gredraw who));
Gint ginqtextfacil (Gchar *ws_type, Gtxfac *fac);
Gint ginqpredtextrep (Gchar *ws_type, Gint idx, Gtxbundl *rep);
Gint ginqtextindices (Gint wsid, Gintlist *idxlist);
Gint ginqtextrep (Gint wsid, Gint idx, Gqtype type, Gtxbundl *rep);
Gint ginqavailwstypes (Gstrlist *wstypes);
Gint ginqdisplayspacesize (Gchar *ws_type, Gdspsize *dspsz);
Gint ginqwscategory (Gchar *ws_type, Gwscat *cat);
Gint ginqwsclass (Gchar *ws_type, Gwsclass *class);
Gint ginqmodwsattr (Gchar *ws_type, Gmodws *dyn);
Gint ginqdefdeferst (Gchar *ws_type, Gdefer *def);
Gint ginqmaxwssttables (Gchar *ws_type, Gwstables *tables);
Gint ginqnumavailinput (Gchar *ws_type, Gnumdev *num);
Gint ginqwsconntype (Gint wsid, Gwsct *ct);
Gint ginqwsst (Gint wsid, Gwsstate *state);
Gint ginqwsdeferupdatest (Gint wsid, Gwsdus *du);
Gint ginqopst (Gos *state);
Gint ginqlevelgks (Glevel *lev);
Gint ginqwsmaxnum (Gwsmax *maxws);
Gint ginqopenws (Gintlist *wsids);
Gint ginqactivews (Gintlist *wsids);
Gint ginqprimattr (Gpriattr *primattr);
Gint ginqindivattr (Gindattr *indivattr);
Gint ginqcurntrannum (Gint *tran);
Gint ginqntrannum (Gintlist *tranlist);
Gint ginqntran (Gint num, Gwstran *tran);
Gint ginqclip (Gcliprec *clipping);
Gint ginqmaxntrannum (Gint *maxtran);
Gint ginqwstran (Gint wsid, Gwsti *wstran);
Gint ginqpixelarraydim (Gint wsid, Grect *rect, Gipoint *dim);
Gint ginqpixelarray (Gint wsid, Gpoint *point, Gipoint *dimen, Gpxarray *pxarr);
Gint ginqpixel (Gint wsid, Gpoint *ppoint, Gint *pix);
Gint gwritegksm (Gint wsid, Gint type, Gint length, Gchar *data);
Gint ggetgksm (Gint wsid, Ggksmit *result);
Gint greadgksm (Gint wsid, Gint length, Gchar *record);
Gint ginterpret (Ggksmit *recInfo, Gchar *data);

int g_gdp(Gint npoints, Gpoint *points, Gint function,
                                   Ggdprec *data);

int gerlog_(int *errnr, int *fctid, int *lun);
int gerhnd_(int *errnr, int *fctid, int *errfil);
int ginqavailgdp(Gchar *type,Ggdplist *gdps);
int gprecc(int il,int *ia,int rl,float *ra,int sl,int *lstr,char **str,int  mldr,int *errind,int *ldr,char *datarec);
void perr(int i, char *s);

#if 0
Gint ginqclip (Gcliprec *clipping);
Gint ginqcolourindices(Gint ws_id, Gintlist       *indices);
Gint gsetcolourrep(Gint ws_id,Gint idx, Gcobundl *rep);
Gint gsetcolorrep(Gint ws_id,Gint idx, Gcobundl *rep);



int gopenws(Gint ws_id , const Gchar * connection ,
                                   const Gchar * ws_type);
int gclosews(Gint ws_id);
int gactivatews(Gint ws_id);
int gdeactivatews(Gint ws_id);
int gclearws(Gint ws_id , Gclrflag control_flag);
int gredrawsegws(Gint ws_id);
int gupdatews(Gint ws_id, Gregen regenflag);
int gsetdeferst(Gint ws_id, Gdefmode deferral_mode,
                                   Girgmode regen_mode);
int gpolyline(Gint num_pts, Gpoint *pts);
int gpolymarker(Gint num_pts, Gpoint *pts);
int gfillarea(Gint num_pts, Gpoint *pts);
/*int gcellarray(xgks_Grect *rect, Gipoint *dim, Gint row,
  Gint *colour); */
int gsetlineind(Gint idx);
int gsetlinetype(Gint type);
int gsetlinewidth(Gfloat width);
int gsetlinecolourind(Gint idx);
int gsetmarkerind(Gint idx);
int gsetmarkertype(Gint type);
int gsetmarkersize(Gfloat size);
int gsetmarkercolourind(Gint colour);
int gsettextind(Gint idx);
int gsettextfontprec(Gtxfp *txfp);
int gflushevents(Gint ws_id,Giclass class,Gint dev);
void gescsetbackingstore(Gint ws, Gint i);
void gescsetprogname(Gchar *name);

int  gcloseseg(void);

Gint gsetpatsize(Gpoint *siz);
Gint gsetfillind(Gint idx);
Gint gsetfillstyleind(Gint idx);
Gint gsettextcolourind(Gint colour);
Gint gsettextpath(Gtxpath path);
Gint gsetfillintstyle(Gflinter style);
Gint gsetfillcolourind(Gint colour);
Gint gsetpatrefpt(Gpoint *ref);
Gint gsetpickid(Gint pick_id);
Gint gcreateseg(Gint name);
Gint gdelseg(Gint name);
Gint gsettextalign(Gtxalign *txalign);
Gint grenameseg(Gint old,Gint new);
Gint ginqsegattr(Gsegattr *segattr);
Gint gmessage(Gint ws_id,Gchar *string);
Gint gsetsegattr(Gint name,Gsegattr *segattr);
Gint gtext(Gpoint *at,Gchar *string);
Gint gsetcharexpan(Gfloat expansion);
Gint gsetcharspace(Gfloat spacing);
Gint gsetasf(Gasfs *asf);
Gint gsetlinerep(Gint ws_id,Gint idx,Glnbundl *rep);
Gint gsetmarkerrep(Gint ws_id,Gint idx,Gmkbundl *rep);
Gint gsettextrep(Gint ws_id,Gint idx,Gtxbundl *bundle);
Gint gsetfillrep(Gint ws_id,Gint idx,Gflbundl *rep);
Gint gsetpatrep(Gint ws_id,Gint idx,Gptbundl *rep);
Gint gsetwswindow(Gint ws_id,Glimit *window);
Gint gsetwsviewport(Gint ws_id,Glimit *viewport);
Gint gsetwindow(Gint trans,Glimit *window);
#endif

int  PSmoSetLandscape(Metafile *mf);
Gint gsetpatsize(Gpoint *siz);
Gint gsetpatrefpt(Gpoint *ref);
int  XgksXDrawLines(Display *dpy,Drawable win,GC gc,XPoint *xpts,int npts,int mode);
Gint XgksFindNTransNpts(int num,Gpoint *ndcpts,Gint *ntrans,Gpoint *wcpts);
int  xXgksInqTextExtent(WS_STATE_PTR ws,TEXT_ST *tx,Gpoint ndc_points[5]);
void XgksUpdatePrimiBound(OUT_PRIMI *primi,Glimit *bound);
void XgksProcessPrimi(OUT_PRIMI *primi);
void XgksOutputToWs(OUT_PRIMI *primi);
void XgksMiniMax(Glimit *bound,Gpoint *pt);
void XgksAppendWsPrimi(OUT_PRIMI *primi);
void XgksAppendSegClip(void);
void XgksAppendWsClip(Glimit *rec);
int  xXgksHighLight(WS_STATE_PTR  ws,Gpoint bd[5]);
void XgksGReDrawWs(WS_STATE_PTR ws);
void XgksReDrawSeg(WS_STATE_PTR ws,Gint seg_id);
void XgksReDrawSegWs(WS_STATE_PTR ws);
int  xXgksClearWs(WS_STATE_PTR wk);
void XgksInsertPrimi(OUT_PRIMI **insert_pt,OUT_PRIMI *elm);
void XgksUnpendPendingTrans(WS_STATE_PTR ws);
void XgksReDrawWs(WS_STATE_PTR ws,OUT_PRIMI *primi);
void XgksUpdateWsSegList(WS_STATE_PTR ws);
void XgksReDrawSegWs(WS_STATE_PTR ws);
void XgksWsWinInterset(WS_STATE_PTR ws,Glimit *v,Glimit *clip);
int  xXgksPolyMarker(WS_STATE_PTR ws,PMARK_ST *pmk_ptr);
int  xXgksFillArea(WS_STATE_PTR ws,FILL_AREA_ST *fill_ptr);
int  xXgksCloseWs(WS_STATE_PTR ws);
Gint xXgksOpenWs(WS_STATE_PTR wk);
void XgksDeletePrimi(OUT_PRIMI *head,OUT_PRIMI **insert_pt);
int  xXgksCellarray(WS_STATE_PTR ws,CELL_ARRAY_ST *cell_ptr);
void XgksInitIDev(WS_STATE_ENTRY *ws);
void XgksInitWssTrans(WS_STATE_PTR ws);
void XgksIDevDelete(WS_STATE_ENTRY *ws);
void XgksDeleteAllSeg(WS_STATE_PTR ws);
Gint xXgksOpenWs(WS_STATE_PTR wk);
int  xXgksText(WS_STATE_PTR ws,TEXT_ST *tx);
int  xXgksMesg(WS_STATE_PTR ws,MESG_ST *mesg);
void XgksInitWssPmarkers(WS_STATE_PTR ws);
int  xXgksPolyLine(WS_STATE_PTR ws, PLINE_ST *plin_ptr);
void xXgksUpdateClip(WS_STATE_PTR ws);
void XgksInitWssPlines(WS_STATE_PTR ws);
int  XgksInitWssFillArea(WS_STATE_PTR ws);
void XgksDrawToWs(WS_STATE_PTR ws,OUT_PRIMI *primi);
int  XgksInsertMesgPrimi(WS_STATE_PTR ws,OUT_PRIMI *primi);
void XgksCleanUpWsSegList(WS_STATE_PTR ws);
void XgksInitGksAsf(void);
void XgksInitGksPlines(void);
void XgksInitGksPmarkers(void);
void XgksInitGksFillArea(void);
void XgksInitWssText(WS_STATE_PTR ws);
void XgksInitGksSegments(void);
void XgksInitGksText(void);
void XgksInitGksTrans(void);
void XgksComputeVec(Gpoint *up_vec, Gpoint *base_vec);
int  XgksSIGIO_OFF(Display *dpy);
int  XgksSIGIO_ON(Display *dpy);
int  xXgksXReDrawWs(WS_STATE_PTR    ws);
void XgksXReDrawWs (WS_STATE_PTR    ws);
int  sockspgrp(int fd,pid_t pid);
int  sockasync(int fd, int yes);
int sio_on(void (*handler)());
int sio_off(void);
void XgksIDevDisable(WS_STATE_ENTRY *ws);
void xXgksUpdateTrans(WS_STATE_PTR    ws);
void XgksIProcessXEvent(XEvent *xev,WS_STATE_ENTRY *ws);
void XgksIDevEnable(WS_STATE_ENTRY *ws);
void xXgksSetForeground(Display *dpy, GC gc,unsigned long   fg);
void xXgksSetFillAreaClipRectangles(Display *dpy,GC gc,WS_STATE_PTR ws,XRectangle *rectangle);
void xXgksSetLineAttributes(Display *dpy,GC gc, unsigned int line_width,int line_style,int cap_style,int join_style);
void xXgksSetStipple(Display *dpy,GC gc,Pixmap stipple);
void xXgksSetFillStyle(Display *dpy,GC gc,int fill_style);
void xXgksSetTile(Display *dpy,GC gc,Pixmap tile);
void XcCopy(WS_STATE_PTR toWs,WS_STATE_PTR fromWs);
void xXgksSetDashes(Display *dpy,GC gc,WS_STATE_PTR ws, Gint i);
void xXgksSetPlineClipRectangles(Display *dpy,GC gc,WS_STATE_PTR ws,XRectangle *rectangle);
void XgksDrawMarkers(Display *dpy,Window win,GC gc,XPoint *pe,int n,int type,float s);
void xXgksSetPmarkerClipRectangles(Display *dpy,GC gc,WS_STATE_PTR ws,XRectangle  *rectangle);
void xXgksSetTextClipRectangles(Display *dpy,GC gc,WS_STATE_PTR ws,XRectangle *rectangle);
int  XgksMoDeactivateWs(WS_STATE_PTR    ws);
void XgksProcessPrimi(OUT_PRIMI      *primi);
int  XgksAwaitEvent(WS_STATE_ENTRY **wslist,int nws, double timeout);
int  XgksEnqueueEvent(Gint ws,Gint dev, Giclass class,char *data, int event_id);
int  xXgksSetColourRep(WS_STATE_PTR ws,int idx,Gcobundl *rep);
int  xXgksInqColourRep(WS_STATE_PTR ws,int idx,Gqtype type, Gcobundl *rep);
int  XgksMaxColours(char *server);
void XgksReDrawSegWs(WS_STATE_PTR ws);
void XgksUpdateWsClip(WS_STATE_PTR ws,Glimit *bound);
int  XgksChoUpdatePrompt(WS_STATE_ENTRY *ws,INPUT_DEV *idev,PromptStatus pstate,XMotionEvent *xmev,int event_id);
int  XgksLocUpdatePrompt(WS_STATE_ENTRY *ws,INPUT_DEV *idev,PromptStatus pstate,Gpoint *newdcpt,XMotionEvent *xmev,int event_id);
int  XgksPicUpdatePrompt(WS_STATE_ENTRY *ws,INPUT_DEV *idev,Gpoint *newdcpt,XMotionEvent *xmev,int event_id);
int  XgksStrUpdatePrompt(WS_STATE_ENTRY *ws,INPUT_DEV *idev,PromptStatus pstate,XKeyPressedEvent *xev,int event_id);
int  XgksStkUpdatePrompt(WS_STATE_ENTRY *ws,INPUT_DEV *idev,PromptStatus pstate,Gpoint *newdcpt,XMotionEvent *xmev,int event_id);
int  XgksValUpdatePrompt(WS_STATE_ENTRY *ws,INPUT_DEV *idev,PromptStatus pstate,Gpoint *newdcpt,XMotionEvent *xmev, int event_id);
void XgksPicDelete(WS_STATE_ENTRY *ws,INPUT_DEV *idev);
void XgksLocDelete(WS_STATE_ENTRY *ws,INPUT_DEV *idev);
void XgksStrDelete(WS_STATE_ENTRY *ws,INPUT_DEV *idev);
void XgksStkDelete(WS_STATE_ENTRY *ws,INPUT_DEV *idev);
void XgksChoDelete(WS_STATE_ENTRY *ws,INPUT_DEV *idev);
void XgksValDelete(WS_STATE_ENTRY *ws,INPUT_DEV *idev);
int  xXgksInqPixelarrayDim(WS_STATE_PTR ws,Grect *rect,Gipoint *dim);
int  xXgksInqPixel(WS_STATE_PTR ws,Gpoint *ppoint,Gint *pix);
int  xXgksInqPixelarray(WS_STATE_PTR ws,Gpoint *point,Gipoint *dim, Gpxarray *pxarr);
Bool XgksFindNTrans(Gpoint *ndcpt,Gloc *gloc);
void XgksProcessClip(Glimit *rec);
void XgksSetLineAttrMo(WS_STATE_PTR ws,Glnattr *lnattr);
void XgksSetMarkAttrMo(WS_STATE_PTR ws,Gmkattr *mkattr);
void XgksSetTextAttrMo(WS_STATE_PTR ws,Gtxattr *txattr,CHATTR *chattr);
void XgksSetFillPatAttrMo(WS_STATE_PTR ws,Gflattr *flattr,PTATTR *ptattr);
    
#endif	/* XGKS_GKS_IMPLEM_H not defined */
